#include "Asteroids.h"

// Constructor
Asteroids::Asteroids() {
	asteroidCount = 0;
}

// Destructor
Asteroids::~Asteroids() {

	// delete the asteroids
	asteroids.start();

	while (asteroids.next())
		delete asteroids.get();
}

void Asteroids::update(float timeScale) {

	// update all asteroids
	asteroids.start();

	while (asteroids.next()) {
		asteroids.get()->update(timeScale);
	}

	// clean asteroids that are done
	asteroids.start();

	while (asteroids.next()) {

		if (asteroids.get()->isRemovable()) {
			delete asteroids.get();
			asteroids.remove();
		}
	}

	starfield1.update(timeScale);
	starfield2.update(timeScale);
}

void Asteroids::paint() {

	starfield1.paint();
	starfield2.paint();
	asteroids.start();

	while (asteroids.next())
		asteroids.get()->paint();
}

void Asteroids::addAsteroid(Ship *ship) {

	Asteroid *newAsteroid = new Asteroid();
	asteroids.add(newAsteroid);

	// Spawn the asteroids away from the ship
	while (newAsteroid->collides(ship->bound)) {
		newAsteroid->pickRandomLocation();
	}
}

// This is for adding asteroids that have been pre-constructed (by the server, for instance)
void Asteroids::addAsteroid(Asteroid *asteroid) {
	asteroids.add(asteroid);
}

void Asteroids::asteroidHit(Asteroid *a, WorldObject *collider, int gameLevel) {

	a->isHit();

	// create asteroidlets
	int parentSize = a->asteroidSize;
	int textureOffset = 0;

	// If the asteroid is the smallest size, it just goes away
	if (parentSize == SMALL) {
		return;
	}

	int numOfAsteroidlets = 0;
	float newAsteroidSize = 5;

	switch (parentSize) {
		case LARGE: 
			numOfAsteroidlets = 2; 
			newAsteroidSize = 32;
			textureOffset = 3;
				break;
		case MED: 			
			numOfAsteroidlets = 4; 
			newAsteroidSize = 16;
			textureOffset = 6;
				break;
	}

	// Throttle the gameLevel variable - used for adding extra asteroid velocity
	if (gameLevel > 6) 
		gameLevel = 6 + (float(gameLevel) / 5.0f);

	for (int i=0; i < numOfAsteroidlets; i++) {

		Asteroid *newAsteroid = new Asteroid();

		newAsteroid->x = a->x; newAsteroid->y = a->y;
		newAsteroid->asteroidSize = parentSize + 1;
		newAsteroid->size = newAsteroidSize;
		newAsteroid->bound.setBoundingCircle(newAsteroid, newAsteroidSize);
		newAsteroid->texture = rand() % 3 + textureOffset;
		newAsteroid->asteroidID = asteroidCount++;

		// Add the velocity of the object that collided with the parent and the parent's own velocity
		newAsteroid->velocity.addVector(collider->velocity*.25f);
		newAsteroid->velocity.addVector(a->velocity);
		newAsteroid->velocity.magnitude *= float(gameLevel)/6.0f + ((1.0f - (1.0f/float(numOfAsteroidlets)))*.25f);

		asteroids.add(newAsteroid);
	}
}

void Asteroids::clearAsteroids() {

	asteroids.start();

	while (asteroids.next()) {

		delete asteroids.get();
		asteroids.remove();
	}
}

void Asteroids::removeAsteroid(int asteroidID) {

	asteroids.start();

	while (asteroids.next()) {

		if (asteroidID == asteroids.get()->asteroidID) {
			delete asteroids.get();
			asteroids.remove();
			return;
		}
	}
}
