#include "BoundCircle.h"

bool BoundCircle::collidesWith(BoundCircle &b) {

	float thisX = parent->x, thisY = parent->y;
	float thatX = b.parent->x, thatY = b.parent->y;

	float distance = float(sqrt((thisX-thatX)*(thisX-thatX)+(thisY-thatY)*(thisY-thatY)));
	
	// If the parent object is overlapped, check those bounding boxes as well
	if (parent->isOverlappedX()) {

		float newX = thisX + parent->getOverlapOffsetX();
		float distanceX = float(sqrt((newX-thatX)*(newX-thatX)+(thisY-thatY)*(thisY-thatY)));

		if (distanceX < distance) distance = distanceX;
	}

	if (parent->isOverlappedY()) {

		float newY = thisY + parent->getOverlapOffsetY();
		float distanceY = float(sqrt((thisX-thatX)*(thisX-thatX)+(newY-thatY)*(newY-thatY)));

		if (distanceY < distance) distance = distanceY;
	}

	if (parent->isOverlappedX() && parent->isOverlappedY()) {

		float newX = thisX + parent->getOverlapOffsetX();
		float newY = thisY + parent->getOverlapOffsetY();
		float distanceXY = float(sqrt((newX-thatX)*(newX-thatX)+(newY-thatY)*(newY-thatY)));

		if (distanceXY < distance) distance = distanceXY;
	}

	if (distance < radius+b.radius)
		return true;

	return false;
};

void BoundCircle::paint(int color) {

	if (!Game::debug) return;

	glDisable(GL_TEXTURE_2D);

	float r, g, b;

	switch (color) {

	case 0: r=0; g=1; b=0; break;
	case 1: r=0; g=0; b=1; break;
	case 2: r=1; g=0; b=0; break;
	case 3: r=1; g=1; b=0; break;

	}

	glColor3f(r, g, b);

	glLineWidth(2);

	glBegin(GL_LINE_LOOP);

	for (int i=0; i < 360; i+=18) {
		float degInRad = i*PI/180; 
		glVertex2f(cos(degInRad)*radius, sin(degInRad)*radius);
	}
	glEnd(); 

	glColor3f(0, 0, 1);
	glBegin(GL_LINE_LOOP);
		glVertex2f(-parent->size, -parent->size);
		glVertex2f(parent->size, -parent->size);
		glVertex2f(parent->size, parent->size);
		glVertex2f(-parent->size, parent->size);
	glEnd(); 
	glEnable(GL_TEXTURE_2D);

	glColor3f(1, 1, 1);
}

void BoundCircle::setBoundingCircle(WorldObject *parent, float radius) { 
	this->parent = parent; 
	this->radius = radius; 
	//inCollision = false;
};

