#include "Special.h"

bool Special::textureLoaded = false;
CL_OpenGLSurface *Special::specialTexture;
 
// Constructor
Special::Special() {
	timeBetweenAppearences = 12;
	reset();
	appearenceLength = 5;
	onScreen = false;

	orbitTheta = 0;
	x = 0; y = 0;
	size = 17;
	bound.setBoundingCircle(this, size);
	heading = -90;

	explosionParticles.setBaseObject(this);

	if (!textureLoaded) {
		specialTexture = new CL_OpenGLSurface("Graphics/powerup", WorldObject::manager);
		textureLoaded = true;
	}
}

// Destructor
Special::~Special() {

}

void Special::update(float timeScale) {

	timeElapsed += timeScale;

	if (timeElapsed > lastAppearence + timeBetweenAppearences) {

		if (!onScreen) newSpecial();
		onScreen = true;

		if (timeElapsed > lastAppearence + timeBetweenAppearences + appearenceLength) {
			onScreen = false;
			lastAppearence = timeElapsed;
		}
	}

	explosionParticles.update(timeScale);
};

void Special::paint() {

	if (onScreen) {
                WorldObject::glState->set_active();
		// Paint the block
		specialTexture->bind();
		paintOverlap();

		// Paint the orbiting particle
		StarfieldParticleContainer::particle->bind();
		orbitTheta += float(PI)/45;

		int size = 12;

		for (int i=0; i < 8; i++) {

			if (i == 3 || i == 7) continue;

			glLoadIdentity();

			glTranslatef(SCREENWIDTH/2+x, SCREENHEIGHT/2+y, 0);
			glRotatef(orbitTheta*180/float(PI), 0, 0, 1);
			glRotatef(45*i, 0, 0, 1);
			glTranslatef(25, 0, 0);

			glColor3f(0, 1, 0);

			if (i == 1 || i == 5)
				glColor3f(0, 0, 1);

			glBegin(GL_QUADS);									
				glTexCoord2f(0,0); glVertex2f(-size, -size);
				glTexCoord2f(1,0); glVertex2f(size, -size);
				glTexCoord2f(1,1); glVertex2f(size, size);
				glTexCoord2f(0,1); glVertex2f(-size, size);
			glEnd();
		}

		glColor3f(1, 1, 1);
	}

	if (!explosionParticles.hasNoAnimations())
		explosionParticles.paint();
};

void Special::isHit() {};
bool Special::isExpired() { return false; };
bool Special::isRemovable() { return false; };

void Special::newSpecial() {
	timeBetweenAppearences = 15 + rand() % 10;
	orbitTheta = 0;

	// Make sure the powerups spawn somewhere where they won't hit the ship
	do {
		x = rand() % SCREENWIDTH - SCREENWIDTH/2; 
		y = rand() % SCREENHEIGHT - SCREENHEIGHT/2; 
	} while (this->collides(gameShip->bound));
}

int Special::checkHit() {

	if (!onScreen) return -1;

	if (gameShip->bound.collidesWith(this->bound)) {

		onScreen = false;
		lastAppearence = timeElapsed;

		// Send back a random powerup
		return rand() % NUM_POWERUP_TYPES;
	}

	return -1;
}

void Special::reset() {
	timeElapsed = 0;
	lastAppearence = 0;
}
