#include "StarfieldParticleContainer.h"

CL_OpenGLSurface *StarfieldParticleContainer::particle;
TexturePtr StarfieldParticleContainer::specialTexture[2];
bool StarfieldParticleContainer::textureLoaded = false;
int StarfieldParticleContainer::instanceCount = 0;

// Constructor
StarfieldParticleContainer::StarfieldParticleContainer() {

	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES; i++) {
		// ================
		// Add new Particle
		// ================

		Particle newParticle;

		float size = rand() % 5 + 5;
		float brightness = size/10;

		// Stars don't really come in green, so scaling the r and g tints together seems to prevent that
		float rTint = float(rand() % 10) / 30.0f;
		float bTint = float(rand() % 10) / 30.0f;

		newParticle.setColor(brightness+rTint, brightness+rTint, brightness+bTint, 1);
		newParticle.setSize(size);

		// I'm going to hijack these variable since they're not used (for twinkling effect)
		newParticle.lifeTimeRemaining = rand() % 360;
		newParticle.totalLife = float(rand() % 10) / 10.0f + .3f;
		if (newParticle.totalLife > 1) newParticle.totalLife = 1;

		// Set start point
		newParticle.setLocation(rand() % SCREENWIDTH - SCREENWIDTH/2, rand() % SCREENHEIGHT  - SCREENHEIGHT/2);

		particles[pointer++] = newParticle;

		if (pointer == USUAL_NUMBER_OF_PARTICLES-1)
			pointer = 0;
	}

	special.setSize(120);
	specialRandomLocation();

	if (!textureLoaded) {
		particle = new CL_OpenGLSurface("Graphics/bullet_particle", WorldObject::manager);
		specialTexture[0] = new CL_OpenGLSurface("Graphics/special_1", WorldObject::manager);
		specialTexture[1] = new CL_OpenGLSurface("Graphics/special_2", WorldObject::manager);
		textureLoaded = true;
	}

	specialType = instanceCount;
	instanceCount++;
}

// Destructor
StarfieldParticleContainer::~StarfieldParticleContainer() {

}

void StarfieldParticleContainer::paint() {

        WorldObject::glState->set_active();

	particle->bind();

	// Base class handles this
	paintParticles();

	paintSpecial();
}

void StarfieldParticleContainer::paintSpecial() {

        WorldObject::glState->set_active();
	specialTexture[specialType]->bind();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(SCREENWIDTH/2+special.x, SCREENHEIGHT/2+special.y, 0);
	glRotatef(specialHeading, 0, 0, 1);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_QUADS);

	glColor4f(1, 1, 1, 1);
								
	glTexCoord2f(0,0); glVertex2f(-special.size, -special.size);
	glTexCoord2f(1,0); glVertex2f(special.size, -special.size);
	glTexCoord2f(1,1); glVertex2f(special.size, special.size);
	glTexCoord2f(0,1); glVertex2f(-special.size, special.size);
		
	glEnd();	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor3f(1, 1, 1);
}

void StarfieldParticleContainer::update(float timeScale) {

	// Twinkling effect
	// Stars don't twinkle it outer space but it looks cool
	for (int i=0; i < USUAL_NUMBER_OF_PARTICLES; i++) {

		if (particles[i].totalLife >= 1) continue;

		particles[i].a = 1 - (sin(particles[i].lifeTimeRemaining)*particles[i].totalLife);

		particles[i].lifeTimeRemaining+=timeScale*5;

	}
}

void StarfieldParticleContainer::specialRandomLocation() {
	specialHeading = rand() % 360;
	special.setLocation(rand() % (SCREENWIDTH-75) - SCREENWIDTH/2, rand() % (SCREENHEIGHT-75)  - SCREENHEIGHT/2);
}
