#include "UFO.h"

bool UFO::textureLoaded = false;
CL_OpenGLSurface *UFO::ufo;
 
// Constructor
UFO::UFO() {

	explosionParticles.setBaseObject(this);

	if (!textureLoaded) {
		ufo = new CL_OpenGLSurface("Graphics/ufo", WorldObject::manager);
		textureLoaded = true;
	}

	size = 32;
	bound.setBoundingCircle(this, size);

	lifetime = 0;
	firingInterval = 3;
}

// Destructor
UFO::~UFO() {

}

void UFO::update(float timeScale) {

	// If the UFO isn't active, we can skip everything but the particles that might be leftover
	if (!onScreen) {
		
		if (!explosionParticles.hasNoAnimations())
			explosionParticles.update(timeScale);

		bulletsFired.start();

		while (bulletsFired.next())
			if (!bulletsFired.get()->bulletParticles.hasNoAnimations()) 
				bulletsFired.get()->update(timeScale);

		return;
	}

	lifetime -= timeScale;

	if (lifetime < 0) {
		onScreen = false; 
		return; 
	}

	x += (velocity*timeScale).getX();
	y += (velocity*timeScale).getY();

	// Offset moves the ship up and down
	y += sin(offsetTheta)*3;
	offsetTheta += float(PI)*timeScale;

	// Fade the ship in
	if (fadeInLevel < 1 && lifetime > 1/fadeInRate)
		fadeInLevel += fadeInRate*timeScale;

	// Fade the ship out
	if (lifetime < 1/fadeInRate)
		fadeInLevel -= fadeInRate*timeScale;

	// Is it time to fire again?
	if (firingInterval < 0) 
		fireAtShip();
	else
		firingInterval -= timeScale;

	// Update the fired bullets
	bulletsFired.start();
	while (bulletsFired.next())
		bulletsFired.get()->update(timeScale);

	// Remove any expired bullets
	bulletsFired.start();
	while (bulletsFired.next()) {

		if (bulletsFired.get()->isRemovable()) {
			delete bulletsFired.get();
			bulletsFired.remove();
		}
	}
};

void UFO::paint() {

	// If the UFO isn't active, we can skip everything but the particles that might be leftover
	if (!onScreen) {

		if (!explosionParticles.hasNoAnimations())
			explosionParticles.paint();

		bulletsFired.start();

		while (bulletsFired.next())
			if (!bulletsFired.get()->bulletParticles.hasNoAnimations()) 
				bulletsFired.get()->paint();

		return;
	}

        WorldObject::glState ->set_active();
	// Paint ufo (with alpha)
	ufo->bind();
	glColor4f(1, 1, 1, fadeInLevel);
	paintOverlap();

	// Paint bullets
	bulletsFired.start();

	while (bulletsFired.next()) {
		glColor4f(1, 0, 0, 1);
		bulletsFired.get()->paint();
	}
};

void UFO::setLocationAndPath() {

	heading = 90;
	offsetTheta = 0;
	fadeInLevel = 0, fadeInRate = 4;
	onScreen = true;

	// Direction 0 - left to right
	// Direction 1 - right to left
	int direction = rand() % 2;

	if (direction == 0) {
		velocity = Vector2(0, 85 + rand() % 45);
		x = -SCREENWIDTH/2+size;
	} else {
		velocity = Vector2(180, 85 + rand() % 45);
		x = SCREENWIDTH/2-size;
	}

	// Keep looping until the location doesn't hit the player
	do {
		y = rand() % SCREENHEIGHT - SCREENHEIGHT/2; 
	} while (this->collides(shipToFireAt->bound));

	// Compute how long the ship will take to get across the screen - so it doesn't wrap around
	lifetime = SCREENWIDTH/abs(velocity.getX());
}

void UFO::fireAtShip() {

	firingInterval = 3;

	// Firing solution
	Vector2 fromUFOToShip;
	fromUFOToShip.setXY(shipToFireAt->x-x, shipToFireAt->y-y);

	// Temporary point the ufo at the ship to fire
	heading = fromUFOToShip.theta + (rand() % 35 - 17);

	// Fire - red color
	bulletsFired.add(new Bullet(this, 1, .8f, .8f));

	// Reset the heading (so it doesn't spin)
	heading = 90;
}

void UFO::isHit() {

	// UFO goes bye-bye
	onScreen = false;
	explosionParticles.burst(80);
};

bool UFO::isExpired() { return !onScreen; };
bool UFO::isRemovable() { return false; };

void UFO::reset() {
	lifetime = 0;
	onScreen = false;
}
