#include "WorldObject.h"

CL_ResourceManager *WorldObject::manager;
CL_OpenGLState *WorldObject::glState;

WorldObject::WorldObject() {
	x = 0; y = 0;
}

WorldObject::~WorldObject() {}

// Is the object hanging off the screen on either axis?
bool WorldObject::isOverlappedY() {

	float yLocation = float(SCREENHEIGHT)/2.0f+y;

	if (yLocation < 0 || yLocation > SCREENHEIGHT)
		y += getOverlapOffsetY();

	return (yLocation - size < 0 || yLocation + size > SCREENHEIGHT) ? true : false;
}

bool WorldObject::isOverlappedX() {

	float xLocation = float(SCREENWIDTH)/2.0f+x;

	if (xLocation < 0 || xLocation > SCREENWIDTH)
		x += getOverlapOffsetX();

	return (xLocation - size < 0 || xLocation + size > SCREENWIDTH) ? true : false;
}

// If it is, how far?
float WorldObject::getOverlapOffsetY() {
	return (float(SCREENHEIGHT)/2.0f+y < SCREENHEIGHT/2) ? SCREENHEIGHT : -SCREENHEIGHT;
}

float WorldObject::getOverlapOffsetX() {
	return (float(SCREENWIDTH)/2.0f+x < SCREENWIDTH/2) ? SCREENWIDTH : -SCREENWIDTH;
}

bool WorldObject::collides(BoundCircle &bnd) {

	// See if the bounding circles are in collision
	if (bound.collidesWith(bnd) && !this->isExpired()) {
		//bound.inCollision = true;
		//bnd.inCollision = true;
		return true;
	} else
		return false;
}

void WorldObject::paintOverlap() {
	if (isExpired()) return;

	bool paintedOverlapX = false;
	bool paintedOverlapY = false;
	bool paintedOverlapXY = false;
	int paintCount = 0;

	float overlapOffsetX = 0, overlapOffsetY = 0;

Paint:

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(SCREENWIDTH/2+x+overlapOffsetX, SCREENHEIGHT/2+y+overlapOffsetY, 0);
	glRotatef(heading+90, 0, 0, 1);

	glBegin(GL_QUADS);									
		glTexCoord2f(0,0); glVertex2f(-size, -size);
		glTexCoord2f(1,0); glVertex2f(size, -size);
		glTexCoord2f(1,1); glVertex2f(size, size);
		glTexCoord2f(0,1); glVertex2f(-size, size);
	glEnd();

	// For debugging, it paints the bounding circles
	bound.paint(paintCount);
	paintCount++;

	// See if the object is overlapped on each of the remaining 3 sides. 
	// If neccesary, compute the offsets and repaint.
	overlapOffsetX = 0, overlapOffsetY = 0;

	if (!paintedOverlapX && isOverlappedX()) {
		overlapOffsetX = getOverlapOffsetX();
		paintedOverlapX = true;
		goto Paint;
	}

	if (!paintedOverlapY && isOverlappedY()) {
		overlapOffsetY = getOverlapOffsetY();
		paintedOverlapY = true;
		goto Paint;
	}

	if (!paintedOverlapXY && isOverlappedX() && isOverlappedY()) {
		overlapOffsetX = getOverlapOffsetX();
		overlapOffsetY = getOverlapOffsetY();
		paintedOverlapXY = true;
		goto Paint;
	}

	glColor3f(1, 1, 1);
}
